/*
 * Copyright (C) 2021 eichehome
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package utils;

/**
 * Diese Klasse stellt eine FIFO-Pipe dar.
 *
 * @author eichehome
 */
public class FifoPipe {

    PipeElement firstElement;
    PipeElement lastElement;

    /**
     * Diese Funktion dient zum Einreihen von Elementen in die FIFO-Pipe
     *
     * @param element ein Element, welches eingereiht werden soll
     */
    public void setElement(Message element) {
        PipeElement pipeElement = new PipeElement(element);
        try {
            lastElement.setNextElement(pipeElement);
            lastElement = pipeElement;
        } catch (NullPointerException ex) {
            firstElement = pipeElement;
            lastElement = pipeElement;
        }
    }

    /**
     * Diese funktion gibt das erste Element der FIFO-Pipe zurück
     *
     * @return Erstes Element der FIFO-Pipe
     */
    public Message getNextElement() {
        Message result = null;
        if (firstElement == null) {
            return null;
        } else {
            if (firstElement == lastElement) {
                result = firstElement.getData();
                firstElement = null;
                lastElement = null;
                return result;
            } else {
                result = firstElement.getData();
                PipeElement next = firstElement.getNext();
                firstElement = next;
                return result;
            }
        }
    }

}
